unit Crop;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, AVICap, Clipbrd, StdCtrls, ComCtrls, ExtCtrls;

const MinWidth=361;
      MinHeight=137;

type
  TCropForm = class(TForm)
    PreviewPanel: TPanel;
    ETop: TEdit;
    CTop: TUpDown;
    ELeft: TEdit;
    CLeft: TUpDown;
    BSample: TButton;
    BClose: TButton;
    BReset: TButton;
    LLeft: TLabel;
    LTop: TLabel;
    LBottom: TLabel;
    EBottom: TEdit;
    CBottom: TUpDown;
    LRight: TLabel;
    ERight: TEdit;
    CRight: TUpDown;
    Preview: TPaintBox;
    procedure BCloseClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure BSampleClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure PreviewPaint(Sender: TObject);
    procedure CropChange(Sender: TObject; Button: TUDBtnType);
    procedure BResetClick(Sender: TObject);
  private
    { Private declarations }
    OverheadX,OverheadY:integer;
    Original,Cropped:TBitmap;
    BaseCaption:string;
    procedure CropIt;
  public
    { Public declarations }
    procedure SampleFrame;
  end;

var
  CropForm: TCropForm;

var FClipRect:TRect;
function ClipRect(Width,Height:integer):TRect;

implementation
uses Main;

{$R *.dfm}

function ClipRect(Width,Height:integer):TRect;
begin with FClipRect do begin
  if (Left+Right<Width) then begin
    Result.Left:=Left;
    Result.Right:=Width-Right;
  end else begin
    Result.Left:=0;
    Result.Right:=Width;
  end;
  if (Top+Bottom<Height) then begin
    Result.Top:=Top;
    Result.Bottom:=Height-Bottom;
  end else begin
    Result.Top:=0;
    Result.Bottom:=Height;
  end;
end; end;

procedure TCropForm.SampleFrame;
var i:integer;
begin
  if MainForm.hCap<>0 then begin
    capGrabFrameNoStop(MainForm.hCap);
    capEditCopy(MainForm.hCap);
  end;
  // Frame wieder zurckholen
  if not Assigned(Original) then Original:=TBitmap.Create;
  try
    Original.LoadFromClipboardFormat(
             cf_Bitmap,Clipboard.GetAsHandle(cf_Bitmap),0);
  except
    Original.Width:=640;
    Original.Height:=480;
  end;
  Original.PixelFormat:=pf24bit;
  i:=Original.Width+OverheadX; if i<MinWidth then i:=MinWidth; Width:=i;
  i:=Original.Height+OverheadY; if i<MinHeight then i:=MinHeight; Height:=i;
  CLeft.Max:=Original.Width;
  CRight.Max:=Original.Width;
  CTop.Max:=Original.Height;
  CBottom.Max:=Original.Height;
  CropIt;
end;

procedure TCropForm.CropIt;
var Rect:TRect;
    Width,Height:integer;
begin
  FClipRect:=Classes.Rect(CLeft.Position,CTop.Position,
                          CRight.Position,CBottom.Position);
  if not Assigned(Original) then exit;
  Width:=Original.Width;
  Height:=Original.Height;
  Rect:=ClipRect(Width,Height);
  with Rect do
    Caption:=Format(BaseCaption,[Width,Height,Right-Left,Bottom-Top]);
  Cropped.Assign(Original);
  with Cropped.Canvas do begin
    Brush.Style:=bsSolid;
    Pen.Mode:=pmNot;
    Rectangle(0,0,Width,Height);
    Rectangle(Rect);
  end;
  Preview.Invalidate;
end;

procedure TCropForm.BCloseClick(Sender: TObject);
begin
  Close;
end;

procedure TCropForm.FormCreate(Sender: TObject);
begin
  // Konfiguration laden
  with Main.INI do begin
    CLeft.Position:=ReadInteger('Crop','Left',0);
    CRight.Position:=ReadInteger('Crop','Right',0);
    CTop.Position:=ReadInteger('Crop','Top',0);
    CBottom.Position:=ReadInteger('Crop','Bottom',0);
  end;
  // Setup
  Preview.ControlStyle:=Preview.ControlStyle+[csOpaque];
  BaseCaption:=Caption+': %dx%d -> %dx%d';
  OverheadX:=Width-Preview.Width;
  OverheadY:=Height-Preview.Height;
  Original:=nil; Cropped:=TBitmap.Create;
  CropIt;
end;

procedure TCropForm.BSampleClick(Sender: TObject);
begin
  SampleFrame;
end;

procedure TCropForm.FormDestroy(Sender: TObject);
begin
  if Assigned(Original) then Original.Free;
  Cropped.Free;
end;

procedure TCropForm.PreviewPaint(Sender: TObject);
begin
  Preview.Canvas.Draw(0,0,Cropped);
end;

procedure TCropForm.CropChange(Sender: TObject; Button: TUDBtnType);
begin
  CropIt;
end;

procedure TCropForm.BResetClick(Sender: TObject);
begin
  CLeft.Position:=0;
  CRight.Position:=0;
  CTop.Position:=0;
  CBottom.Position:=0;
  CropIt;
end;

initialization
  FClipRect:=Rect(0,0,0,0);
end.
