unit Main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, AVICap, Menus, Clipbrd, JPEG, INIFiles;

const CapID=$7475;
      SnapDistance=10;
      INIFileName='VidGrab.ini';
type TPreviewMode=(pmNone,pmPreview,pmOverlay);

type
  TMainForm = class(TForm)
    Menu: TMainMenu;
    MFile: TMenuItem;
    MSettings: TMenuItem;
    MEdit: TMenuItem;
    MHelp: TMenuItem;
    MCapSource: TMenuItem;
    MCapFormat: TMenuItem;
    MCapDisplay: TMenuItem;
    MView: TMenuItem;
    MOverlay: TMenuItem;
    MPreview: TMenuItem;
    N1: TMenuItem;
    MOrigSize: TMenuItem;
    MHalfOrigSize: TMenuItem;
    MFullscreen: TMenuItem;
    MQuit: TMenuItem;
    N2: TMenuItem;
    MCopyFrame: TMenuItem;
    N3: TMenuItem;
    MFrameToClipboard: TMenuItem;
    MFrameToFile: TMenuItem;
    MFrameToBoth: TMenuItem;
    N4: TMenuItem;
    MAlwaysOnTop: TMenuItem;
    MSetPostprocess: TMenuItem;
    MCrop: TMenuItem;
    N5: TMenuItem;
    MOpen: TMenuItem;
    MFileOptions: TMenuItem;
    OpenDialog: TSaveDialog;
    MAbout: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure MCapSourceClick(Sender: TObject);
    procedure MCapFormatClick(Sender: TObject);
    procedure MCapDisplayClick(Sender: TObject);
    procedure MViewClick(Sender: TObject);
    procedure PreviewModeClick(Sender: TObject);
    procedure MOrigSizeClick(Sender: TObject);
    procedure MHalfOrigSizeClick(Sender: TObject);
    procedure MFullscreenClick(Sender: TObject);
    procedure MQuitClick(Sender: TObject);
    procedure MCopyFrameClick(Sender: TObject);
    procedure MAlwaysOnTopClick(Sender: TObject);
    procedure MSetPostprocessClick(Sender: TObject);
    procedure MCropClick(Sender: TObject);
    procedure MOpenClick(Sender: TObject);
    procedure MFileOptionsClick(Sender: TObject);
    procedure MAboutClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    { Private declarations }
    FSLeft,FSTop,FSWidth,FSHeight:integer;
    AlwaysOnTop:boolean;
    DestFileBase,DestFileExt:string;
    DestFileJPEG:boolean;
    Serial:integer;
    CurrentFile:string;
    procedure FormMove(var msg:TMessage); message WM_MOVE;
    procedure SetAlwaysOnTop(AOT:boolean);
    procedure NextFile;
    procedure BuildFileName;
    procedure UpdateCaption;
  public
    { Public declarations }
    hCap:THandle;
    PreviewMode:TPreviewMode;
    DestFileDigits:integer;
    JPEGQuality:integer; JPEGProgressive:boolean;
    BaseDir:string;
    procedure SetPreviewMode(AMode:TPreviewMode);
  end;

var
  MainForm: TMainForm;

var INI:TINIFile;  

implementation
uses Process, ProcSet, Crop, FileOpt, About;

{$R *.dfm}

procedure TMainForm.FormCreate(Sender: TObject);
var DeviceID:integer;
begin
  randomize;
  BaseDir:=IncludeTrailingPathDelimiter(GetCurrentDir);
  FSWidth:=-1;
  PreviewMode:=pmNone;
  // Config lesen
  INI:=TINIFile.Create(BaseDir+INIFileName);
  with INI do begin
    DeviceID:=ReadInteger('Settings','DeviceID',0);
    AlwaysOnTop:=ReadBool('Settings','AlwaysOnTop',false);
    SetCurrentDir(ReadString('Output','LastDir','.'));
    DestFileBase:=ReadString('Output','Base','grab');
    DestFileExt:=ReadString('Output','Ext','.bmp');
    DestFileJPEG:=ReadBool('Output','JPEG',false);
    DestFileDigits:=ReadInteger('Output','Digits',4);
    Serial:=ReadInteger('Output','Serial',0);
    JPEGQuality:=ReadInteger('Output','JPEGQuality',75);
    JPEGProgressive:=ReadBool('Output','JPEGProgressive',false);
  end;
  OpenDialog.InitialDir:=GetCurrentDir;
  SetAlwaysOnTop(AlwaysOnTop);
  BuildFileName; UpdateCaption;
  // Capture initialisieren
  hCap:=capCreateCaptureWindowA('VidGrab',WS_CHILD OR WS_VISIBLE,0,0,384,288,
        Handle,CapID);
  if hCap=0 then
    MessageDlg('Konnte Capture-Window nicht erzeugen!',mtError,[mbOK],0)
  else begin
    capDriverConnect(hCap,DeviceID);
    capPreviewScale(hCap,1);
    capPreviewRate(hCap,40);
    capOverlay(hCap,1);
    SetPreviewMode(pmOverlay);
    MOrigSizeClick(nil);
  end;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  if hCap<>0 then begin
    capDriverDisconnect(hCap);
    DestroyWindow(hCap);
  end;
  with TINIFile.Create(BaseDir+INIFileName) do begin
    WriteBool('Settings','AlwaysOnTop',AlwaysOnTop);
    WriteString('Output','LastDir',GetCurrentDir);
    WriteString('Output','Base',DestFileBase);
    WriteString('Output','Ext',DestFileExt);
    WriteBool('Output','JPEG',DestFileJPEG);
    WriteInteger('Output','Digits',DestFileDigits);
    WriteInteger('Output','Serial',Serial);
    WriteInteger('Output','JPEGQuality',JPEGQuality);
    WriteBool('Output','JPEGProgressive',JPEGProgressive);
    WriteInteger('Crop','Left',FClipRect.Left);
    WriteInteger('Crop','Right',FClipRect.Right);
    WriteInteger('Crop','Top',FClipRect.Top);
    WriteInteger('Crop','Bottom',FClipRect.Bottom);
    with ProcSetForm do begin
      WriteBool('Processing','MedianX',DirtyINIHack_MedianX);
      WriteBool('Processing','MedianY',DirtyINIHack_MedianY);
      WriteBool('Processing','HalfSize',DirtyINIHack_HalfSize);
      WriteString('Processing','Deinterlace',DirtyINIHack_Deinterlace);
    end;
    Free;
  end;
end;

procedure TMainForm.FormResize(Sender: TObject);
begin
  if hCap=0 then exit;
  SetWindowPos(hCap,0,0,0,ClientWidth,ClientHeight,SWP_NOZORDER OR SWP_NOMOVE);
end;

procedure TMainForm.MCapSourceClick(Sender: TObject);
begin
  if hCap<>0 then capDlgVideoSource(hCap);
end;

procedure TMainForm.MCapFormatClick(Sender: TObject);
begin
  if hCap<>0 then capDlgVideoFormat(hCap);
end;

procedure TMainForm.MCapDisplayClick(Sender: TObject);
begin
  if hCap<>0 then capDlgVideoDisplay(hCap);
end;

procedure TMainForm.MViewClick(Sender: TObject);
begin
  MOverlay.Checked:=(PreviewMode=pmOverlay);
  MPreview.Checked:=(PreviewMode=pmPreview);
  MAlwaysOnTop.Checked:=AlwaysOnTop;
end;

procedure TMainForm.SetPreviewMode(AMode:TPreviewMode);
begin
  if hCap=0 then exit;
  if PreviewMode=pmOverlay then capOverlay(hCap,0);
  if PreviewMode=pmPreview then capPreview(hCap,0);
  PreviewMode:=AMode;
  if PreviewMode=pmOverlay then capOverlay(hCap,1);
  if PreviewMode=pmPreview then capPreview(hCap,1);
end;

procedure TMainForm.PreviewModeClick(Sender: TObject);
var Mode:TPreviewMode;
begin
  Mode:=TPreviewMode((Sender as TComponent).Tag);
  if PreviewMode=Mode then SetPreviewMode(pmNone)
                      else SetPreviewMode(Mode);
end;

procedure TMainForm.MOrigSizeClick(Sender: TObject);
var Status:TCapStatus;
begin
  if hCap=0 then exit;
  if FSWidth>=0 then MFullscreenClick(nil);
  if SendMessage(hCap,WM_CAP_GET_STATUS,sizeof(Status),LPARAM(@Status))=0
     then exit;
  ClientWidth:=Status.uiImageWidth;
  ClientHeight:=Status.uiImageHeight;
end;

procedure TMainForm.MHalfOrigSizeClick(Sender: TObject);
var Status:TCapStatus;
begin
  if hCap=0 then exit;
  if FSWidth>=0 then MFullscreenClick(nil);
  if SendMessage(hCap,WM_CAP_GET_STATUS,sizeof(Status),LPARAM(@Status))=0
     then exit;
  ClientWidth:=Status.uiImageWidth DIV 2;
  ClientHeight:=Status.uiImageHeight DIV 2;
end;

procedure TMainForm.MFullscreenClick(Sender: TObject);
var i:integer;
begin
  if FSWidth<0 then begin
    FSLeft:=Left; FSTop:=Top;
    FSWidth:=Width; FSHeight:=Height;
    for i:=Menu.Items.Count-1 downto 0 do
      Menu.Items[i].Visible:=false;
    SetWindowLong(Handle,GWL_STYLE,GetWindowLong(Handle,GWL_STYLE)
      AND not(WS_CAPTION) AND not(WS_THICKFRAME));
    SetWindowPos(Handle,HWND_TOPMOST,0,0,0,0,SWP_NOSIZE OR SWP_NOMOVE);
    Left:=0; Top:=0;
    Width:=Screen.Width;
    Height:=Screen.Height;
  end else begin
    for i:=Menu.Items.Count-1 downto 0 do
      Menu.Items[i].Visible:=true;
    SetWindowLong(Handle,GWL_STYLE,GetWindowLong(Handle,GWL_STYLE)
      OR WS_CAPTION OR WS_THICKFRAME);
    Left:=FSLeft; Top:=FSTop;
    Width:=FSWidth; Height:=FSHeight;
    SetAlwaysOnTop(AlwaysOnTop);
    FSWidth:=-1;
  end;
end;

procedure TMainForm.MQuitClick(Sender: TObject);
begin
  Close;
end;

procedure TMainForm.MCopyFrameClick(Sender: TObject);
var Mode,Index:integer;
    BMP:TBitmap; JPEG:TJPEGImage;
    AFormat:word; AData:THandle; APalette:HPalette;
begin
  if hCap=0 then exit;
  Mode:=(Sender as TComponent).Tag;
  capGrabFrameNoStop(hCap);
  capEditCopy(hCap);
  if Mode=0 then exit;
  // Frame wieder zurckholen
  BMP:=TBitmap.Create;
  try
    BMP.LoadFromClipboardFormat(cf_Bitmap,Clipboard.GetAsHandle(cf_Bitmap),0);
  except
    BMP.Free;
    exit;
  end;
  // Postprocessing
  BMP.PixelFormat:=pf24bit;
  with Crop.FClipRect do
    if (Left<>0) OR (Right<>0) OR (Top<>0) OR (Bottom<>0) then
      with ClipRect(BMP.Width,BMP.Height) do begin
        BMP.Canvas.Draw(-Left,-Top,BMP);
        BMP.Width:=Right-Left;
        BMP.Height:=Bottom-Top;
      end;
  for Index:=1 to NumFilters do
    FilterChain[Index](BMP);
  // in ZA kopieren
  if (Mode AND 1)<>0 then begin
    BMP.SaveToClipboardFormat(AFormat,AData,APalette);
    Clipboard.SetAsHandle(AFormat,AData);
    if APalette<>0 then DeleteObject(APalette);
  end;
  // speichern
  if (Mode AND 2)<>0 then begin
    NextFile;
    JPEG:=nil;
    try
      if DestFileJPEG then begin
        JPEG:=TJPEGImage.Create;
        with JPEG do begin
          Assign(BMP);
          CompressionQuality:=JPEGQuality;
          ProgressiveEncoding:=JPEGProgressive;
          SaveToFile(CurrentFile);
        end;
      end else
        BMP.SaveToFile(CurrentFile);
    except
      if Assigned(JPEG) then JPEG.Free;
    end;
  end;
  BMP.Free;
end;

procedure TMainForm.SetAlwaysOnTop(AOT:boolean);
begin
  if AOT then SetWindowPos(Handle,HWND_TOPMOST,0,0,0,0,SWP_NOSIZE OR SWP_NOMOVE)
         else SetWindowPos(Handle,HWND_NOTOPMOST,0,0,0,0,SWP_NOSIZE OR SWP_NOMOVE);
  AlwaysOnTop:=AOT;
end;

procedure TMainForm.MAlwaysOnTopClick(Sender: TObject);
begin
  if FSWidth<0 then SetAlwaysOnTop(not(AlwaysOnTop));
end;

procedure TMainForm.MSetPostprocessClick(Sender: TObject);
begin
  ProcSetForm.Show;
end;

procedure TMainForm.MCropClick(Sender: TObject);
begin
  CropForm.SampleFrame;
  CropForm.Show;
end;

procedure TMainForm.MOpenClick(Sender: TObject);
begin
  if not OpenDialog.Execute then exit;
  DestFileBase:=OpenDialog.FileName;
  DestFileExt:=ExtractFileExt(DestFileBase);
  SetLength(DestFileBase,length(DestFileBase)-length(DestFileExt));
  while ((length(DestFileBase)>0) AND
         (DestFileBase[length(DestFileBase)] IN ['0'..'9']))
         do SetLength(DestFileBase,length(DestFileBase)-1);
  DestFileJPEG:=(LowerCase(DestFileExt)='.jpg') OR
                (LowerCase(DestFileExt)='.jpeg');
  Serial:=0;
  BuildFileName; UpdateCaption;
end;

procedure TMainForm.BuildFileName;
var t:string; i:integer;
begin
  str(Serial:DestFileDigits,t);
  for i:=length(t) downto 1 do
    if t[i]=#32 then t[i]:=#48;
  CurrentFile:=DestFileBase+t+DestFileExt;
end;

procedure TMainForm.UpdateCaption;
begin
  Caption:='VidGrab - '+CurrentFile;
end;

procedure TMainForm.MFileOptionsClick(Sender: TObject);
begin
  if FileOptForm.ShowModal=mrOK then begin
    BuildFileName; UpdateCaption;
  end;
end;

procedure TMainForm.NextFile;
begin
  repeat
    inc(Serial);
    BuildFileName;
  until not FileExists(CurrentFile);
  UpdateCaption;
end;

procedure TMainForm.FormMove(var msg:TMessage);
begin
  if abs(Left+Width-Screen.Width)<SnapDistance then Left:=Screen.Width-Width;
  if abs(Top+Height-Screen.Height)<SnapDistance then Top:=Screen.Height-Height;
  if abs(Left)<SnapDistance then Left:=0;
  if abs(Top)<SnapDistance then Top:=0;
  msg.Result:=0;
end;

procedure TMainForm.MAboutClick(Sender: TObject);
begin
  AboutForm.ShowModal;
end;

procedure TMainForm.FormShow(Sender: TObject);
begin
  if Assigned(INI) then FreeAndNil(INI);
end;

end.
