unit ProcSet;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Process;

type
  TProcSetForm = class(TForm)
    GFilters: TGroupBox;
    FMedianX: TCheckBox;
    FMedianY: TCheckBox;
    FHalfSize: TCheckBox;
    GDeinterlace: TGroupBox;
    BClose: TButton;
    DNone: TRadioButton;
    DReduceY: TRadioButton;
    DMedian: TRadioButton;
    DKeepTop: TRadioButton;
    DKeepBottom: TRadioButton;
    DAverage: TRadioButton;
    procedure BCloseClick(Sender: TObject);
    procedure FilterChainChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  ProcSetForm: TProcSetForm;

const MaxFilters=8;
var FilterChain:array[1..MaxFilters]of TProcessingFilter;
    NumFilters:integer;

var DirtyINIHack_MedianX:boolean;
    DirtyINIHack_MedianY:boolean;
    DirtyINIHack_HalfSize:boolean;
    DirtyINIHack_Deinterlace:string;

implementation
uses Main;

{$R *.dfm}

procedure AddFilter(Filter:TProcessingFilter);
begin
  if NumFilters>=MaxFilters then exit;
  inc(NumFilters);
  FilterChain[NumFilters]:=Filter;
end;

procedure TProcSetForm.BCloseClick(Sender: TObject);
begin
  Close;
end;

procedure TProcSetForm.FilterChainChange(Sender: TObject);
begin
  NumFilters:=0;
  if FMedianX.Checked then AddFilter(Process.MedianX);
  if DKeepTop.Checked then AddFilter(Process.KeepTopDeinterlace);
  if DKeepBottom.Checked then AddFilter(Process.KeepBottomDeinterlace);
  if DAverage.Checked then AddFilter(Process.AveragingDeinterlace);
  if DMedian.Checked OR
    (FMedianY.Checked AND not(DNone.Checked)) then AddFilter(Process.MedianY);
  if FHalfSize.Checked then AddFilter(Process.ReduceX);
  if FHalfSize.Checked OR DReduceY.Checked then AddFilter(Process.ReduceY);
end;

procedure TProcSetForm.FormCreate(Sender: TObject);
var s:string;
begin
  with Main.INI do begin
    FMedianX.Checked:=ReadBool('Processing','MedianX',true);
    FMedianY.Checked:=ReadBool('Processing','MedianY',true);
    FHalfSize.Checked:=ReadBool('Processing','HalfSize',false);
    s:=LowerCase(ReadString('Processing','Deinterlace',''));
         if s='none' then DNone.Checked:=true
    else if s='median' then DMedian.Checked:=true
    else if s='top' then DKeepTop.Checked:=true
    else if s='bottom' then DKeepBottom.Checked:=true
    else if s='reduce' then DReduceY.Checked:=true
    else if s='average' then DAverage.Checked:=true;
  end;
  FilterChainChange(nil);
end;

procedure TProcSetForm.FormDestroy(Sender: TObject);
begin
  DirtyINIHack_MedianX:=FMedianX.Checked;
  DirtyINIHack_MedianY:=FMedianY.Checked;
  DirtyINIHack_HalfSize:=FHalfSize.Checked;
       if DNone.Checked then DirtyINIHack_Deinterlace:='None'
  else if DMedian.Checked then DirtyINIHack_Deinterlace:='Median'
  else if DKeepTop.Checked then DirtyINIHack_Deinterlace:='Top'
  else if DKeepBottom.Checked then DirtyINIHack_Deinterlace:='Bottom'
  else if DReduceY.Checked then DirtyINIHack_Deinterlace:='Reduce'
  else if DAverage.Checked then DirtyINIHack_Deinterlace:='Average';
end;

initialization
  NumFilters:=0;
end.
