unit Process;
interface
uses Graphics;

type TProcessingFilter=procedure(BMP:TBitmap);

procedure MedianX(BMP:TBitmap);
procedure MedianY(BMP:TBitmap);

procedure Deinterlace(BMP:TBitmap; KeepLine:integer);
procedure KeepTopDeinterlace(BMP:TBitmap);
procedure KeepBottomDeinterlace(BMP:TBitmap);
procedure AveragingDeinterlace(BMP:TBitmap);

procedure ReduceX(BMP:TBitmap);
procedure ReduceY(BMP:TBitmap);

implementation
uses SysUtils;

function Med(a,b,c:integer):integer;
begin
  if a<b then begin
    if b<c then Result:=b else
    if a<c then Result:=c else
                Result:=a;
  end else begin
    if b>c then Result:=b else
    if a<c then Result:=a else
                Result:=c;
  end;
end;

procedure MedianX(BMP:TBitmap);
var X,Y,MaxX:integer;
    L:PByteArray; CP,NP:^byte;
    LR,LG,LB,CR,CG,CB,NR,NG,NB:integer;
begin
  MaxX:=BMP.Width-3;
  for Y:=(BMP.Height-1) downto 0 do begin
    L:=BMP.ScanLine[Y];
    LB:=L[0]; LG:=L[1]; LR:=L[2];
    CB:=L[3]; CG:=L[4]; CR:=L[5];
    CP:=@L[3]; NP:=@L[6];
    for X:=MaxX downto 0 do begin
      NB:=NP^; CP^:=Med(LB,CB,NB); LB:=CB; CB:=NB; inc(CP); inc(NP);
      NG:=NP^; CP^:=Med(LG,CG,NG); LG:=CG; CG:=NG; inc(CP); inc(NP);
      NR:=NP^; CP^:=Med(LR,CR,NR); LR:=CR; CR:=NR; inc(CP); inc(NP);
    end;
  end;
end;

procedure MedianY(BMP:TBitmap);
var X,Y,MaxX:integer;
    L:PByteArray; LP,CP,NP:^byte;
begin
  MaxX:=BMP.Width-1;
  for Y:=(BMP.Height-2) downto 1 do begin
    L:=BMP.ScanLine[Y-1]; LP:=@L[0];
    L:=BMP.ScanLine[Y]; CP:=@L[0];
    L:=BMP.ScanLine[Y+1]; NP:=@L[0];
    for X:=MaxX downto 0 do begin
      CP^:=Med(LP^,CP^,NP^); inc(LP); inc(CP); inc(NP);
      CP^:=Med(LP^,CP^,NP^); inc(LP); inc(CP); inc(NP);
      CP^:=Med(LP^,CP^,NP^); inc(LP); inc(CP); inc(NP);
    end;
  end;
end;

procedure Deinterlace(BMP:TBitmap; KeepLine:integer);
var X,Y,MaxX,MaxY:integer;
    L:PByteArray; LP,CP,NP:^byte;
begin
  MaxX:=BMP.Width-1;
  MaxY:=BMP.Height-2;
  Y:=(KeepLine AND 1)+1;
  while (Y<=MaxY) do begin
    L:=BMP.ScanLine[Y-1]; LP:=@L[0];
    L:=BMP.ScanLine[Y]; CP:=@L[0];
    L:=BMP.ScanLine[Y+1]; NP:=@L[0];
    for X:=MaxX downto 0 do begin
      CP^:=(LP^+NP^) SHR 1; inc(LP); inc(CP); inc(NP);
      CP^:=(LP^+NP^) SHR 1; inc(LP); inc(CP); inc(NP);
      CP^:=(LP^+NP^) SHR 1; inc(LP); inc(CP); inc(NP);
    end;
    inc(Y,2);
  end;
end;

procedure KeepTopDeinterlace(BMP:TBitmap);    begin Deinterlace(BMP,0); end;
procedure KeepBottomDeinterlace(BMP:TBitmap); begin Deinterlace(BMP,1); end;

procedure AveragingDeinterlace(BMP:TBitmap);
var X,Y,MaxX,MaxY:integer;
    L,M:PByteArray; P1,P2,PD:^byte;
begin
  MaxX:=BMP.Width-1;
  MaxY:=(BMP.Height SHR 1)-1;
  for Y:=0 to MaxY do begin
    L:=BMP.ScanLine[Y SHL 1]; P1:=@L[0]; PD:=@L[0];
    L:=BMP.ScanLine[(Y SHL 1) OR 1]; P2:=@L[0];
    for X:=MaxX downto 0 do begin
      PD^:=(P1^+P2^) SHR 1; inc(P1); inc(P2); inc(PD);
      PD^:=(P1^+P2^) SHR 1; inc(P1); inc(P2); inc(PD);
      PD^:=(P1^+P2^) SHR 1; inc(P1); inc(P2); inc(PD);
    end;
  end;
  Deinterlace(BMP,0);
  L:=BMP.ScanLine[BMP.Height-2];
  M:=BMP.ScanLine[BMP.Height-1];
  Move(L[0],M[0],BMP.Width*3);
end;

procedure ReduceX(BMP:TBitmap);
var X,Y,MaxX:integer;
    L:PByteArray; P1,P2,PD:^byte;
begin
  MaxX:=(BMP.Width SHR 1)-1;
  for Y:=(BMP.Height-1) downto 0 do begin
    L:=BMP.ScanLine[Y];
    P1:=@L[0]; P2:=@L[3]; PD:=@L[0];
    for X:=MaxX downto 0 do begin
      PD^:=(P1^+P2^) SHR 1; inc(PD); inc(P1);   inc(P2);
      PD^:=(P1^+P2^) SHR 1; inc(PD); inc(P1);   inc(P2);
      PD^:=(P1^+P2^) SHR 1; inc(PD); inc(P1,4); inc(P2,4);
    end;
  end;
  BMP.Width:=MaxX+1;
end;

procedure ReduceY(BMP:TBitmap);
var X,Y,MaxX,MaxY:integer;
    L:PByteArray; P1,P2,PD:^byte;
begin
  MaxX:=BMP.Width-1;
  MaxY:=(BMP.Height SHR 1)-1;
  for Y:=0 to MaxY do begin
    L:=BMP.ScanLine[Y SHL 1]; P1:=@L[0];
    L:=BMP.ScanLine[(Y SHL 1) OR 1]; P2:=@L[0];
    L:=BMP.ScanLine[Y]; PD:=@L[0];
    for X:=MaxX downto 0 do begin
      PD^:=(P1^+P2^) SHR 1; inc(P1); inc(P2); inc(PD);
      PD^:=(P1^+P2^) SHR 1; inc(P1); inc(P2); inc(PD);
      PD^:=(P1^+P2^) SHR 1; inc(P1); inc(P2); inc(PD);
    end;
  end;
  BMP.Height:=MaxY+1;
end;

end.
